﻿using System;
using System.Drawing;
using System.Runtime.InteropServices;

namespace TestProject
{
    /// <summary>
    /// PICTURE 변환자
    /// </summary>
    public static class PictureConverter
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Class
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 그림 설명 - PICTDESC

        /// <summary>
        /// 그림 설명
        /// </summary>
        private static class PICTDESC
        {
            //////////////////////////////////////////////////////////////////////////////////////////////////// Field
            ////////////////////////////////////////////////////////////////////////////////////////// Static
            //////////////////////////////////////////////////////////////////////////////// Public

            #region Field

            /// <summary>
            /// PICTYPE_UNINITIALIZED
            /// </summary>
            public const short PICTYPE_UNINITIALIZED = -1;

            /// <summary>
            /// PICTYPE_NONE
            /// </summary>
            public const short PICTYPE_NONE = 0;

            /// <summary>
            /// PICTYPE_BITMAP
            /// </summary>
            public const short PICTYPE_BITMAP = 1;

            /// <summary>
            /// PICTYPE_METAFILE
            /// </summary>
            public const short PICTYPE_METAFILE = 2;

            /// <summary>
            /// PICTYPE_ICON
            /// </summary>
            public const short PICTYPE_ICON = 3;

            /// <summary>
            /// PICTYPE_ENHMETAFILE
            /// </summary>
            public const short PICTYPE_ENHMETAFILE = 4;

            #endregion

            //////////////////////////////////////////////////////////////////////////////////////////////////// Class
            ////////////////////////////////////////////////////////////////////////////////////////// Public

            #region 아이콘 - Icno

            /// <summary>
            /// 아이콘
            /// </summary>
            [StructLayout(LayoutKind.Sequential)]
            public class Icon
            {
                //////////////////////////////////////////////////////////////////////////////////////////////////// Field
                ////////////////////////////////////////////////////////////////////////////////////////// Public

                #region Field

                /// <summary>
                /// 크기
                /// </summary>
                public int Size = Marshal.SizeOf(typeof(PICTDESC.Icon));

                /// <summary>
                /// 그림 타입
                /// </summary>
                public int PictureType = PICTDESC.PICTYPE_ICON;

                /// <summary>
                /// 아이콘 핸들
                /// </summary>
                public IntPtr IconHandle = IntPtr.Zero;

                /// <summary>
                /// 미사용 1
                /// </summary>
                public int Unused1 = 0;

                /// <summary>
                /// 미사용 2
                /// </summary>
                public int Unused2 = 0;

                #endregion

                //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
                ////////////////////////////////////////////////////////////////////////////////////////// Public

                #region 생성자 - Icon(icon)

                /// <summary>
                /// 생성자
                /// </summary>
                /// <param name="icon">아이콘</param>
                public Icon(System.Drawing.Icon icon)
                {
                    IconHandle = icon.ToBitmap().GetHicon();
                }

                #endregion
            }

            #endregion
            #region 비트맵 - Bitmap

            /// <summary>
            /// 비트맵
            /// </summary>
            [StructLayout(LayoutKind.Sequential)]
            public class Bitmap
            {
                //////////////////////////////////////////////////////////////////////////////////////////////////// Field
                ////////////////////////////////////////////////////////////////////////////////////////// Public

                #region Field

                /// <summary>
                /// 크기
                /// </summary>
                public int Size = Marshal.SizeOf(typeof(PICTDESC.Bitmap));

                /// <summary>
                /// 그림 타입
                /// </summary>
                public int PictureType = PICTDESC.PICTYPE_BITMAP;

                /// <summary>
                /// 비트맵 핸들
                /// </summary>
                public IntPtr BitmapHandle = IntPtr.Zero;

                /// <summary>
                /// 팔레트 핸들
                /// </summary>
                public IntPtr PaletteHandle = IntPtr.Zero;

                /// <summary>
                /// 미사용
                /// </summary>
                public int Unused = 0;

                #endregion

                //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
                ////////////////////////////////////////////////////////////////////////////////////////// Public

                #region 생성자 - Bitmap(bitmap)

                /// <summary>
                /// 생성자
                /// </summary>
                public Bitmap(System.Drawing.Bitmap bitmap)
                {
                    BitmapHandle = bitmap.GetHbitmap();
                }

                #endregion
            }

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Import
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region OLE 그림 생성하기 (간접) - OleCreatePictureIndirect(pictureDescription, interfaceID, own);

        /// <summary>
        /// OLE 그림 생성하기 (간접)
        /// </summary>
        /// <param name="pictureDescription">그림 설명</param>
        /// <param name="interfaceID">인터페이스 ID</param>
        /// <param name="own">소유 여부</param>
        /// <returns>IPictureDisp 인터페이스 객체</returns>
        [DllImport("OleAut32.dll", EntryPoint = "OleCreatePictureIndirect", ExactSpelling = true, PreserveSig = false)]
        private static extern stdole.IPictureDisp OleCreatePictureIndirect([MarshalAs(UnmanagedType.AsAny)] object pictureDescription, ref Guid interfaceID, bool own);

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// IPictureDisp 인터페이스 GUID
        /// </summary>
        private static Guid _iPictureDispGUID = typeof(stdole.IPictureDisp).GUID;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region IPictureDisp 인터페이스 객체 구하기 - ToPicture(icon)

        /// <summary>
        /// IPictureDisp 인터페이스 객체 구하기
        /// </summary>
        /// <param name="icon">아이콘</param>
        /// <returns>IPictureDisp 인터페이스 객체</returns>
        public static stdole.IPictureDisp ToPicture(Icon icon)
        {
            PICTDESC.Icon pictDescIcon = new PICTDESC.Icon(icon);

            return OleCreatePictureIndirect(pictDescIcon, ref _iPictureDispGUID, true);
        }

        #endregion
        #region IPictureDisp 인터페이스 객체 구하기 - ToPicture(image)

        /// <summary>
        /// IPictureDisp 인터페이스 객체 구하기
        /// </summary>
        /// <param name="image">이미지</param>
        /// <returns>IPictureDisp 인터페이스 객체</returns>
        public static stdole.IPictureDisp ToPicture(Image image)
        {
            Bitmap bitmap = (image is Bitmap) ? (Bitmap)image : new Bitmap(image);

            PICTDESC.Bitmap pictDescBitmap = new PICTDESC.Bitmap(bitmap);

            return OleCreatePictureIndirect(pictDescBitmap, ref _iPictureDispGUID, true);
        }

        #endregion
    }
}